==> xwidgetsbase-6.5.1.tar.gz: lib/XWidgets/XItemsComplexWidgets/XItem.dart <==
part of xwidgetbase;

class XItem extends StatelessWidget {
  final bool titleOnTop;
  final String? title;
  final Color? titleColor;
  final Color? titleBorderColor;
  final TextStyle? titleStyle;

  ///Modificherai il widget dentro il TitleContainer senza toccare il widget che fa da contenitore
  final Widget? titleWidget;
  final TextAlign titleTextAlign;
  final double titleContainerWidth;
  final double? titleContainerHeight;

  ///Allineamento del Title solo se la Label è sopra
  final AlignmentGeometry? titleAlignOnTop;

  ///Allineamento del Title solo se la Label è a sinistra
  final AlignmentGeometry? titleAlignOnLeft;
  final Color? titleBackGroundColor;
  final double? titleWidthBorder;
  final Color? colorCard;

  ///Allineamento orizzontale dell'XItem
  final CrossAxisAlignment? crossAxisAlignmentXItem;

  ///Allineamento Verticale dell'XItem
  final MainAxisAlignment? mainAxisAlignmentXItem;
  final List<Widget>? children;
  // final bool visibleBottomBorderOfCard;
  final CrossAxisAlignment? childrensCrossAxisAlignment;
  final BoxConstraints? constraintsOfXItem;
  final Axis? wrapDirection;
  final double? wrapSpacing;
  final double? wrapRunSpacing;
  final double? wrapHeight;

  final double? wrapWidth;

  ///Utilizzato principalmente su APP
  final Function()? onTap;
  XItem({
    Key? key,
    this.onTap,
    this.wrapSpacing,
    this.wrapWidth,
    this.wrapHeight,
    this.wrapRunSpacing,
    this.wrapDirection,
    this.titleAlignOnTop,
    this.mainAxisAlignmentXItem,
    this.titleWidget,
    this.crossAxisAlignmentXItem,
    this.colorCard,
    this.titleStyle,
    this.titleAlignOnLeft,
    // this.visibleBottomBorderOfCard__ = false,
    this.titleColor,
    this.titleBorderColor,
    this.title = "",
    this.titleContainerWidth = 150,
    this.titleContainerHeight,
    this.children,
    this.childrensCrossAxisAlignment,
    this.titleOnTop = false,
    this.titleTextAlign = TextAlign.center,
    this.titleBackGroundColor,
    this.titleWidthBorder,
    double? width,
    double? height,
    BoxConstraints? constraintsOfXItem,
  })  : constraintsOfXItem = (width != null || height != null) ? constraintsOfXItem?.tighten(width: width, height: height) ?? BoxConstraints.tightFor(width: width, height: height) : constraintsOfXItem,
        super(key: key);

  static Widget? ThreeConcatenated_XItem<T, K>({
    required double height,
    required Axis wrapDirection,
    required double fontSizeForItem,
    required String title,
    required Color? titleColor,
    required Iterable<MapEntry<dynamic, List<T>>> items,
    required Widget Function(T) titleFirstWidget,
    required K Function(T) groupListsBy_secondItems,
    required dynamic Function(T) titleSecondWidget,
    required String titleSecondXItem,
    required Widget Function(T) thirdWidget,
    bool groupByForLastLevel = false, //TODO
    K Function(T)? groupListsBy_LastItems, //TODO
    Widget Function(MapEntry<K, List<T>>)? thirdWidgetForGroupBy, //TODO
    bool firstTitleOnTop = true,
    bool secondTitleOnTop = false,
    bool? borderAllAroundList,
    Color? backGroundColorTitleContainer,
    Color? colorBorderOfarea,
    Color? backGroundColor,
    Color? backGroundColorFirstXItem,
    Color? backGroundColorSecondXItem,
    double? labelContainerHeight,
    double? titleContainerHeight,
    double? fontSizeForTitleOfArea,
    double? widthFirstXItem,
    double? widthSecondXItem,
    double? firstTitleContainerWidth,
    double? secondTitleContainerWidth,
    Axis? firstWrapDirection,
    Axis? secondWrapDirection,
    Axis? thirdWrapDirection,
    MapEntry<dynamic, List<T>>? titleFirstWidgetForGroupBy,
  }) {
    var xi = XItem(
        title: title,
        titleColor: colorBorderOfarea ?? Colors.white,
        titleOnTop: true,
        titleBackGroundColor: backGroundColorTitleContainer ?? null,
        titleContainerHeight: titleContainerHeight ?? 40,
        titleWidthBorder: 5,
        titleAlignOnTop: Alignment.center,
        titleContainerWidth: double.infinity,
        titleStyle: TextStyle(color: titleColor, fontSize: fontSizeForTitleOfArea ?? 24, fontWeight: FontWeight.bold),
        colorCard: backGroundColor,
        children: [
          items.length == 0
              ? XItemWidget(
                  borderWidth: 5,
                  titleBackColor: colorBorderOfarea ?? Colors.black,
                  borderAll: borderAllAroundList ?? true,
                  child: Container(
                    height: 100,
                    child: Row(children: [Expanded(child: Text("..nessuna..", textAlign: TextAlign.center, style: XStyles.xStyleText(fontSize: 30)))]),
                  ))
              : XItemWidget(
                  alignmentChild: Alignment.topLeft,
                  heightChildContainer: height,
                  borderWidth: 5,
                  titleBackColor: colorBorderOfarea ?? Colors.black,
                  child: Wrap(
                      spacing: 1,
                      runSpacing: 6,
                      direction: wrapDirection,
                      //1ST GRP - DATA ANNO
                      children: items.map((grp1) {
                        var xii = XItem(
                            wrapDirection: firstWrapDirection ?? Axis.vertical,
                            width: widthFirstXItem ?? 400,
                            height: null,
                            wrapSpacing: 1,
                            wrapRunSpacing: 1,
                            //1ST GROUP
                            titleWidget: grp1.value.map(titleFirstWidget).first,
                            colorCard: backGroundColorFirstXItem ?? Colors.blue.withOpacity(0.2),
                            crossAxisAlignmentXItem: CrossAxisAlignment.start,
                            mainAxisAlignmentXItem: MainAxisAlignment.start,
                            titleAlignOnTop: firstTitleOnTop != false ? Alignment.centerLeft : null,
                            titleContainerWidth: firstTitleOnTop != false ? double.infinity - 20 : firstTitleContainerWidth ?? 80,
                            titleOnTop: firstTitleOnTop,
                            titleStyle: TextStyle(color: colorBorderOfarea ?? titleColor, fontSize: 20),
                            // 2nd GROUP
                            children: grp1.value.groupListsBy<K>(groupListsBy_secondItems).entries.map((grp2) {
                              return Container(
                                  child: XItem(
                                wrapDirection: secondWrapDirection ?? Axis.horizontal,
                                width: widthSecondXItem ?? 200,
                                title: grp2.value.map(titleSecondWidget).first.toString(),
                                titleColor: XColors.xcolor_ForegroundLight,
                                titleAlignOnTop: secondTitleOnTop != false ? Alignment.centerLeft : null,
                                titleContainerWidth: secondTitleOnTop == false ? secondTitleContainerWidth ?? 50 : double.infinity - 20,
                                titleOnTop: secondTitleOnTop,
                                titleBackGroundColor: Colors.transparent,
                                colorCard: backGroundColorSecondXItem,
                                // 3th GROUP
                                children: [
                                  XItemWidget(
                                      title: "",
                                      borderAll: true,
                                      titleBackColor: Colors.transparent,
                                      backGroundColorInternalWidget: Colors.transparent,
                                      child: Wrap(
                                        direction: thirdWrapDirection ?? Axis.horizontal,
                                        children: groupByForLastLevel == false
                                            ? grp2.value.map(thirdWidget).toList()
                                            : //
                                            grp2.value.groupListsBy<K>(groupListsBy_LastItems!).entries.map(thirdWidgetForGroupBy!).toList(),
                                        // :

                                        // ///
                                        // grp2.value.groupListsBy<K>(groupListsBy_LastItems).entries.map(thirdWidget).toList(),
                                      )),
                                ],
                              ));
                            }).toList());
                        return xii;
                      }).toList())),
        ]);
    return xi;
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
        onTap: onTap,
        child: ConstrainedBox(
          constraints: constraintsOfXItem ?? BoxConstraints(),
          child: Container(
            child: Card(
              color: colorCard ?? Colors.transparent,
              child: Container(
                  // decoration: visibleBottomBorderOfCard == true ? BoxDecoration(border: Border(bottom: BorderSide(width: 6, color: Colors.blue.withOpacity(0.5)))) : null,
                  child: Column(
                mainAxisSize: MainAxisSize.min,
                mainAxisAlignment: mainAxisAlignmentXItem ?? MainAxisAlignment.center,
                crossAxisAlignment: crossAxisAlignmentXItem ?? CrossAxisAlignment.center,
                children: [
                  !titleOnTop || title == null
                      ? Container()
                      : Container(
                          alignment: titleAlignOnTop ?? Alignment.center,
                          width: titleContainerWidth,
                          height: titleContainerHeight,
                          color: titleBackGroundColor != null ? null : titleBackGroundColor ?? Colors.transparent,
                          margin: EdgeInsets.fromLTRB(0, 0, 0, 0),
                          decoration: titleBackGroundColor != null
                              ? BoxDecoration(
                                  color: titleBackGroundColor ?? Colors.transparent,
                                  border: Border(
                                    top: BorderSide(color: titleBorderColor ?? Colors.transparent, width: titleWidthBorder ?? 2),
                                    left: BorderSide(color: titleBorderColor ?? Colors.transparent, width: titleWidthBorder ?? 2),
                                    right: BorderSide(color: titleBorderColor ?? Colors.transparent, width: titleWidthBorder ?? 2),
                                    // bottom: BorderSide(color: colortitle ?? Colors.transparent, width: widthBorderTitleContainer ?? 2),
                                  ))
                              : null,
                          child: titleWidget ??
                              Container(
                                alignment: titleAlignOnTop,
                                height: titleContainerHeight,
                                child: Text(title!, textAlign: titleTextAlign, style: titleStyle ?? XStyles.xStyTextForLabel(titleColor)),
                              )),
                  Container(
                    child: Row(children: [
                      titleOnTop || title == null
                          ? Container()
                          : Container(
                              alignment: titleAlignOnLeft ?? Alignment.center,
                              width: titleContainerWidth,
                              height: titleContainerHeight,
                              padding: EdgeInsets.symmetric(vertical: 2),
                              color: titleBackGroundColor ?? Colors.transparent,
                              margin: EdgeInsets.fromLTRB(0, 0, 0, 0),
                              child: titleWidget ??
                                  Container(
                                    height: titleContainerHeight,
                                    child: Text(
                                      title!,
                                      textAlign: titleTextAlign,
                                      style: titleStyle ?? XStyles.xStyTextForLabel(titleColor),
                                    ),
                                  )),
                      Flexible(
                          child: Wrap(
                        crossAxisAlignment: WrapCrossAlignment.center,
                        direction: wrapDirection ?? Axis.horizontal,
                        spacing: wrapSpacing ?? 5,
                        runSpacing: wrapRunSpacing ?? 5,
                        children: children!,
                      ))
                    ]),
                  )
                ],
              )),
            ),
          ),
        ));
  }
}

==> xwidgetsbase-6.5.7.tar.gz: lib/XWidgets/XItemsComplexWidgets/XItem.dart <==
part of xwidgetbase;

class XItem extends StatelessWidget {
  final bool titleOnTop;
  final String? title;
  final Color? titleColor;
  final Color? titleBorderColor;
  final TextStyle? titleStyle;

  ///Modificherai il widget dentro il TitleContainer senza toccare il widget che fa da contenitore
  final Widget? titleWidget;
  final TextAlign titleTextAlign;
  final double titleContainerWidth;
  final double? titleContainerHeight;

  ///Allineamento del Title solo se la Label è sopra
  final AlignmentGeometry? titleAlignOnTop;

  ///Allineamento del Title solo se la Label è a sinistra
  final AlignmentGeometry? titleAlignOnLeft;
  final Color? titleBackGroundColor;
  final double? titleWidthBorder;
  final Color? colorCard;

  ///Allineamento orizzontale dell'XItem
  final CrossAxisAlignment? crossAxisAlignmentXItem;

  ///Allineamento Verticale dell'XItem
  final MainAxisAlignment? mainAxisAlignmentXItem;
  final List<Widget>? children;
  // final bool visibleBottomBorderOfCard;
  final CrossAxisAlignment? childrensCrossAxisAlignment;
  final BoxConstraints? constraintsOfXItem;
  final Axis? wrapDirection;
  final double? wrapSpacing;
  final double? wrapRunSpacing;
  final double? wrapHeight;

  final double? wrapWidth;

  ///Utilizzato principalmente su APP
  final Function()? onTap;
  XItem({
    Key? key,
    this.onTap,
    this.wrapSpacing,
    this.wrapWidth,
    this.wrapHeight,
    this.wrapRunSpacing,
    this.wrapDirection,
    this.titleAlignOnTop,
    this.mainAxisAlignmentXItem,
    this.titleWidget,
    this.crossAxisAlignmentXItem,
    this.colorCard,
    this.titleStyle,
    this.titleAlignOnLeft,
    // this.visibleBottomBorderOfCard__ = false,
    this.titleColor,
    this.titleBorderColor,
    this.title = "",
    this.titleContainerWidth = 150,
    this.titleContainerHeight,
    this.children,
    this.childrensCrossAxisAlignment,
    this.titleOnTop = false,
    this.titleTextAlign = TextAlign.center,
    this.titleBackGroundColor,
    this.titleWidthBorder,
    double? width,
    double? height,
    BoxConstraints? constraintsOfXItem,
  })  : constraintsOfXItem = (width != null || height != null) ? constraintsOfXItem?.tighten(width: width, height: height) ?? BoxConstraints.tightFor(width: width, height: height) : constraintsOfXItem,
        super(key: key);

  static Widget? Three_ConcatenatedXItem<T, K>({
    required double height,
    required Axis wrapDirection,
    required double fontSizeForItem,
    required String title,
    required Color? titleColor,
    required Iterable<MapEntry<dynamic, List<T>>> items,
    required Widget Function(T) titleFirstWidget,
    required K Function(T) firstGroupListsBy,
    required dynamic Function(T) titleSecondWidget,
    required String titleSecondXItem,
    required Widget Function(T) thirdWidget,
    bool groupByForLastLevel = false, //TODO
    K Function(T)? groupListsBy_LastItems, //TODO
    Widget Function(MapEntry<K, List<T>>)? thirdWidgetForGroupBy, //TODO
    bool titleFirstOnTop = true,
    bool titleSecondOnTop = false,
    bool? borderAllAroundList,
    Color? backGroundColorTitleContainer,
    Color? colorBorderOfarea,
    Color? backGroundColor,
    Color? backGroundColorFirstXItem,
    Color? backGroundColorSecondXItem,
    double? titleContainerHeight,
    double? fontSizeForTitleOfArea,
    double? widthFirstXItem,
    double? widthSecondXItem,
    double? titleFirstContainerWidth,
    double? titleSecondContainerWidth,
    Axis? firstWrapDirection,
    Axis? secondWrapDirection,
    Axis? thirdWrapDirection,
    MapEntry<dynamic, List<T>>? titleFirstWidgetForGroupBy,
  }) {
    var xi = XItem(
        title: title,
        titleColor: colorBorderOfarea ?? Colors.white,
        titleOnTop: true,
        titleBackGroundColor: backGroundColorTitleContainer ?? null,
        titleContainerHeight: titleContainerHeight ?? 40,
        titleWidthBorder: 5,
        titleAlignOnTop: Alignment.center,
        titleContainerWidth: double.infinity,
        titleStyle: TextStyle(color: titleColor, fontSize: fontSizeForTitleOfArea ?? 24, fontWeight: FontWeight.bold),
        colorCard: backGroundColor,
        children: [
          items.length == 0
              ? XItemWidget(
                  borderWidth: 5,
                  titleBackColor: colorBorderOfarea ?? Colors.black,
                  borderAll: borderAllAroundList ?? true,
                  child: Container(
                    height: 100,
                    child: Row(children: [Expanded(child: Text("..nessuna..", textAlign: TextAlign.center, style: XStyles.xStyleText(fontSize: 30)))]),
                  ))
              : XItemWidget(
                  alignmentChild: Alignment.topLeft,
                  heightChildContainer: height,
                  borderWidth: 5,
                  titleBackColor: colorBorderOfarea ?? Colors.black,
                  child: Wrap(
                      spacing: 1,
                      runSpacing: 6,
                      direction: wrapDirection,
                      //1ST GRP - DATA ANNO
                      children: items.map((grp1) {
                        var xii = XItem(
                            wrapDirection: firstWrapDirection ?? Axis.vertical,
                            width: widthFirstXItem ?? 400,
                            height: null,
                            wrapSpacing: 1,
                            wrapRunSpacing: 1,
                            //1ST GROUP
                            titleWidget: grp1.value.map(titleFirstWidget).first,
                            colorCard: backGroundColorFirstXItem ?? Colors.blue.withOpacity(0.2),
                            crossAxisAlignmentXItem: CrossAxisAlignment.start,
                            mainAxisAlignmentXItem: MainAxisAlignment.start,
                            titleAlignOnTop: titleFirstOnTop != false ? Alignment.centerLeft : null,
                            titleContainerWidth: titleFirstOnTop != false ? double.infinity - 20 : titleFirstContainerWidth ?? 80,
                            titleOnTop: titleFirstOnTop,
                            titleStyle: TextStyle(color: colorBorderOfarea ?? titleColor, fontSize: 20),
                            // 2nd GROUP
                            children: grp1.value.groupListsBy<K>(firstGroupListsBy).entries.map((grp2) {
                              return Container(
                                  child: XItem(
                                wrapDirection: secondWrapDirection ?? Axis.horizontal,
                                width: widthSecondXItem ?? 200,
                                title: grp2.value.map(titleSecondWidget).first.toString(),
                                titleColor: XColors.xcolor_ForegroundLight,
                                titleAlignOnTop: titleSecondOnTop != false ? Alignment.centerLeft : null,
                                titleContainerWidth: titleSecondOnTop == false ? titleSecondContainerWidth ?? 50 : double.infinity - 20,
                                titleOnTop: titleSecondOnTop,
                                titleBackGroundColor: Colors.transparent,
                                colorCard: backGroundColorSecondXItem,
                                // 3th GROUP
                                children: [
                                  XItemWidget(
                                      title: "",
                                      borderAll: true,
                                      titleBackColor: Colors.transparent,
                                      backGroundColorInternalWidget: Colors.transparent,
                                      child: Wrap(
                                        direction: thirdWrapDirection ?? Axis.horizontal,
                                        children: groupByForLastLevel == false
                                            ? grp2.value.map(thirdWidget).toList()
                                            : //
                                            grp2.value.groupListsBy<K>(groupListsBy_LastItems!).entries.map(thirdWidgetForGroupBy!).toList(),
                                      )),
                                ],
                              ));
                            }).toList());
                        return xii;
                      }).toList())),
        ]);
    return xi;
  }

  static Widget One_ConcatenatedXItem<T>({
    required double fontSizeForItem,
    required String title,
    required Color? titleColor,
    required List<T> items,
    required Widget Function(T) widgetItemInList,
    double? fontSizeForTitleOfArea,
    Axis? wrapDirectionForList,
    bool titleOnTop = true,
    Color? backGroundColorTitleContainer,
    Color? backGroundColor,
    double? titleContainerHeight,
    double? titleContainerWidth,
    AlignmentGeometry? titleAlignOnTop,
    AlignmentGeometry? titleAlignOnLeft,
  }) {
    return XItem(
        title: title,
        titleColor: titleColor,
        titleOnTop: titleOnTop,
        colorCard: backGroundColor,
        titleWidthBorder: 5,
        titleBackGroundColor: backGroundColorTitleContainer ?? Colors.black,
        titleAlignOnTop: titleAlignOnTop ?? Alignment.center,
        titleAlignOnLeft: titleAlignOnLeft ?? Alignment.centerLeft,
        titleContainerHeight: titleContainerHeight ?? 40,
        titleStyle: TextStyle(color: titleColor, fontSize: fontSizeForTitleOfArea, fontWeight: FontWeight.bold),
        titleContainerWidth: titleOnTop == true ? double.infinity : titleContainerWidth ?? 40,
        children: [
          XItemWidget(
              borderWidth: 5,
              title: "",
              titleBackColor: titleColor,
              child: Wrap(
                spacing: 2,
                runSpacing: 2,
                direction: wrapDirectionForList ?? Axis.horizontal,
                //LIST
                children: items.map(widgetItemInList).toList(),
              ))
        ]);
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
        onTap: onTap,
        child: ConstrainedBox(
          constraints: constraintsOfXItem ?? BoxConstraints(),
          child: Container(
            child: Card(
              color: colorCard ?? Colors.transparent,
              child: Container(
                  // decoration: visibleBottomBorderOfCard == true ? BoxDecoration(border: Border(bottom: BorderSide(width: 6, color: Colors.blue.withOpacity(0.5)))) : null,
                  child: Column(
                mainAxisSize: MainAxisSize.min,
                mainAxisAlignment: mainAxisAlignmentXItem ?? MainAxisAlignment.center,
                crossAxisAlignment: crossAxisAlignmentXItem ?? CrossAxisAlignment.center,
                children: [
                  !titleOnTop || title == null
                      ? Container()
                      : Container(
                          alignment: titleAlignOnTop ?? Alignment.center,
                          width: titleContainerWidth,
                          height: titleContainerHeight,
                          color: titleBackGroundColor != null ? null : titleBackGroundColor ?? Colors.transparent,
                          margin: EdgeInsets.fromLTRB(0, 0, 0, 0),
                          decoration: titleBackGroundColor != null
                              ? BoxDecoration(
                                  color: titleBackGroundColor ?? Colors.transparent,
                                  border: Border(
                                    top: BorderSide(color: titleBorderColor ?? Colors.transparent, width: titleWidthBorder ?? 2),
                                    left: BorderSide(color: titleBorderColor ?? Colors.transparent, width: titleWidthBorder ?? 2),
                                    right: BorderSide(color: titleBorderColor ?? Colors.transparent, width: titleWidthBorder ?? 2),
                                    // bottom: BorderSide(color: colortitle ?? Colors.transparent, width: widthBorderTitleContainer ?? 2),
                                  ))
                              : null,
                          child: titleWidget ??
                              Container(
                                alignment: titleAlignOnTop,
                                height: titleContainerHeight,
                                child: Text(title!, textAlign: titleTextAlign, style: titleStyle ?? XStyles.xStyTextForLabel(titleColor)),
                              )),
                  Container(
                    child: Row(children: [
                      titleOnTop || title == null
                          ? Container()
                          : Container(
                              alignment: titleAlignOnLeft ?? Alignment.center,
                              width: titleContainerWidth,
                              height: titleContainerHeight,
                              padding: EdgeInsets.symmetric(vertical: 2),
                              color: titleBackGroundColor ?? Colors.transparent,
                              margin: EdgeInsets.fromLTRB(0, 0, 0, 0),
                              child: titleWidget ??
                                  Container(
                                    height: titleContainerHeight,
                                    child: Text(
                                      title!,
                                      textAlign: titleTextAlign,
                                      style: titleStyle ?? XStyles.xStyTextForLabel(titleColor),
                                    ),
                                  )),
                      Flexible(
                          child: Wrap(
                        crossAxisAlignment: WrapCrossAlignment.center,
                        direction: wrapDirection ?? Axis.horizontal,
                        spacing: wrapSpacing ?? 5,
                        runSpacing: wrapRunSpacing ?? 5,
                        children: children!,
                      ))
                    ]),
                  )
                ],
              )),
            ),
          ),
        ));
  }
}
